@@ -1,30 +1,30 @@
 use std::collections::HashMap;
 use std::error::Error;
-use std::path::PathBuf;
+use std::path::{Path, PathBuf};
 
 use crossterm::event::{self, Event, KeyCode};
+use quake_core::entry::entry_defines;
 use quake_core::entry::entry_paths::EntryPaths;
 use quake_core::usecases::entry_usecases;
 use quake_core::QuakeConfig;
 use tui::backend::Backend;
+use tui::widgets::ListState;
 use tui::Terminal;
 
 use crate::entry_action::action_result_to_main_widget;
 use crate::ui::draw;
-use crate::widgets::{CmdLine, MainWidget};
+use crate::widgets::MainWidget;
 
 pub struct App {
     pub state: AppState,
     pub config: QuakeConfig,
     pub main_widget: MainWidget,
-    pub cmd_line: CmdLine,
 }
 
 impl App {
     pub fn new(config: QuakeConfig) -> App {
         App {
             main_widget: MainWidget::Home,
-            cmd_line: CmdLine::default(),
             state: Default::default(),
             config,
         }
@@ -59,22 +59,22 @@ impl App {
     }
 
     pub fn message_clear(&mut self) {
-        self.cmd_line.message.clear();
+        self.state.message.clear();
     }
 
     pub fn send_message(&mut self, message: &str) {
         self.message_clear();
-        self.cmd_line.message.push_str(message);
+        self.state.message.push_str(message);
     }
 
     pub fn input_push(&mut self, ch: char) {
         self.state.input.push(ch);
-        self.cmd_line.message.push(ch);
+        self.state.message.push(ch);
     }
 
     pub fn input_pop(&mut self) {
         self.state.input.pop();
-        self.cmd_line.message.pop();
+        self.state.message.pop();
     }
 
     pub fn collect_command(&mut self) -> String {
@@ -98,13 +98,14 @@ impl App {
         match self.state.mode {
             Mode::Normal => match key_code {
                 KeyCode::Char(':') => {
-                    self.collect_command();
-                    self.message_clear();
-                    self.state.mode = Mode::Command;
+                    self.jump_to_command_mode();
                 }
                 KeyCode::Char('i') => {
                     self.state.mode = Mode::Insert;
                 }
+                KeyCode::Char('j') => self.entry_next(),
+                KeyCode::Char('k') => self.entry_prev(),
+                KeyCode::Char('a') => self.action_auto_complete(),
                 _ => {}
             },
             Mode::Command => match key_code {
@@ -151,7 +152,15 @@ impl App {
         let command: String = self.collect_command();
         match command.as_str() {
             "quit" => self.shutdown(),
-            "listAll" => self.main_widget = MainWidget::EntryTypes,
+            "listAll" => {
+                let entry_defines_path =
+                    Path::new(&self.config.workspace).join(EntryPaths::entries_define());
+                let defines = entry_defines::from_path(&entry_defines_path);
+                if !defines.is_empty() {
+                    self.state.entry_list_state.select(Some(0));
+                }
+                self.main_widget = MainWidget::EntryTypes(defines);
+            }
             "save" => self.save_entry(),
             other => {
                 return action_result_to_main_widget(other, &self.config)
@@ -161,12 +170,65 @@ impl App {
 
         Ok(())
     }
+
+    pub fn entry_next(&mut self) {
+        if let MainWidget::EntryTypes(defines) = &self.main_widget {
+            let len = defines.len();
+            let i = match self.state.entry_list_state.selected() {
+                Some(i) => {
+                    if i >= len - 1 {
+                        0
+                    } else {
+                        i + 1
+                    }
+                }
+                None => 0,
+            };
+            self.state.entry_list_state.select(Some(i));
+        }
+    }
+
+    pub fn entry_prev(&mut self) {
+        if let MainWidget::EntryTypes(defines) = &self.main_widget {
+            let len = defines.len();
+            let i = match self.state.entry_list_state.selected() {
+                Some(i) => {
+                    if i == 0 {
+                        len - 1
+                    } else {
+                        i - 1
+                    }
+                }
+                None => 0,
+            };
+            self.state.entry_list_state.select(Some(i));
+        }
+    }
+
+    fn action_auto_complete(&mut self) {
+        if let MainWidget::EntryTypes(defines) = &self.main_widget {
+            if let Some(idx) = self.state.entry_list_state.selected() {
+                let cmd = format!("{}.add: ", defines[idx].entry_type);
+                self.jump_to_command_mode();
+                self.state.input = cmd.clone();
+                self.state.message = cmd;
+            }
+        }
+    }
+
+    fn jump_to_command_mode(&mut self) {
+        self.collect_command();
+        self.message_clear();
+        self.state.mode = Mode::Command;
+    }
 }
 
 pub struct AppState {
     running: bool,
     pub mode: Mode,
     input: String,
+    pub entry_list_state: ListState,
+    pub message: String,
 }
 
 impl Default for AppState {
@@ -175,6 +237,8 @@ impl Default for AppState {
             running: true,
             mode: Mode::Normal,
             input: "".to_string(),
+            entry_list_state: ListState::default(),
+            message: "".to_string(),
         }
     }
 }
@@ -215,7 +279,7 @@ mod tests {
         let command = app.collect_command();
         assert_eq!(command, expect.to_string());
         assert_eq!(app.state.input, "".to_string());
-        assert_eq!(app.cmd_line.message, expect.to_string());
+        assert_eq!(app.state.message, expect.to_string());
     }
 
     #[rstest]
@@ -228,10 +292,10 @@ mod tests {
 
         app.input_push('g');
         app.input_push('t');
-        assert_eq!(app.cmd_line.message, "gt".to_string());
+        assert_eq!(app.state.message, "gt".to_string());
 
         app.send_message(message);
-        assert_eq!(app.cmd_line.message, message.to_string());
+        assert_eq!(app.state.message, message.to_string());
     }
 
     #[rstest]
@@ -243,12 +307,12 @@ mod tests {
         app.handle_key_event(KeyCode::Char('l')).unwrap();
         app.handle_key_event(KeyCode::Char('s')).unwrap();
         assert_eq!(app.state.input, "ls".to_string());
-        assert_eq!(app.cmd_line.message, "ls".to_string());
+        assert_eq!(app.state.message, "ls".to_string());
 
         app.handle_key_event(KeyCode::Esc).unwrap();
         assert_eq!(app.state.mode, Mode::Normal);
         assert_eq!(app.state.input, "".to_string());
-        assert_eq!(app.cmd_line.message, "".to_string());
+        assert_eq!(app.state.message, "".to_string());
     }
 
     #[rstest]
@@ -258,11 +322,11 @@ mod tests {
     #[case("success")]
     fn test_into_command_mode(mut app: App, #[case] message: &str) {
         app.state.input = message.to_string();
-        app.cmd_line.message = message.to_string();
+        app.state.message = message.to_string();
 
         app.handle_key_event(KeyCode::Char(':')).unwrap();
         assert_eq!(app.state.input, "".to_string());
-        assert_eq!(app.cmd_line.message, "".to_string());
+        assert_eq!(app.state.message, "".to_string());
     }
 
     #[rstest]
@@ -301,7 +365,7 @@ mod tests {
         assert_eq!(app.state.mode, Mode::Normal);
 
         let error_message = format!("Unknown command: {}", inputs.iter().collect::<String>());
-        assert_eq!(app.cmd_line.message, error_message);
+        assert_eq!(app.state.message, error_message);
     }
 
     #[rstest]
